Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const mocoolka_tools_io_1 = require("mocoolka-tools-io");
const mocoolka_type_1 = require("mocoolka-type");
const mocoolka_function_1 = require("mocoolka-function");
const mocoolka_algebraic_1 = require("mocoolka-algebraic");
const { StrMap } = mocoolka_algebraic_1.strmap;
const validateInteger = (value) => mocoolka_type_1.isInteger(Number(value)) ? true : 'Please enter a valid integer';
const AppOption = {
    title: 'Welcome to use csvToMongoDB. The is a open source program for import data from csv file to MongoDB.',
    logo: 'Yu Speed',
    file: 'app.json',
    isWatch: true,
    pattern: '',
    isHeader: false,
    fields: 'symbol,date,price,volume,openInterest,askPrice1,askVolume1,bidPrice1,bidVolume1',
    validate(data) {
        return mocoolka_algebraic_1.strmap.toArray(new StrMap(this.params)).reduce(([key, value], pre) => {
            return pre + value['validate'] ? (value['validate'](data[key]) + '\n') : '';
        }, '');
    },
    params: {
        hostname: {
            type: "string",
            description: 'Please enter your monggoDB hostname',
            'default': mocoolka_function_1.constant('localhost'),
            componentType: 'string',
        },
        port: {
            type: "int",
            description: 'Please enter your monggoDB port',
            'default': mocoolka_function_1.constant(27017),
            componentType: 'number',
            validate: validateInteger
        },
        database: {
            type: "string",
            description: 'Please enter your database name',
            'default': mocoolka_function_1.constant('users'),
            componentType: 'string',
        },
        collection: {
            type: "string",
            description: 'Please enter your collection name',
            'default': mocoolka_function_1.constant('ticks'),
            componentType: 'string',
        },
        username: {
            type: "string",
            description: 'Please enter your username',
            'default': mocoolka_function_1.constant('admin'),
            componentType: 'string',
        },
        password: {
            type: "string",
            description: 'Please enter your password',
            'default': mocoolka_function_1.constant('1'),
            componentType: 'password',
        },
        pattern: {
            type: "string",
            description: 'Please enter a import pattern that search csv file.',
            'default': mocoolka_function_1.constant('f:/history/data/tick/*.csv'),
            componentType: 'string',
        },
        mongoimportPath: {
            type: "string",
            description: 'Please enter the shell command mongoimport path',
            'default': mocoolka_function_1.constant('D:/MongoDB/Server/3.4/bin/'),
            componentType: 'string',
            validate(value) {
                if (!mocoolka_tools_io_1.default.fileExist(path.resolve(value, 'mongoimport.exe'))) {
                    return 'mongoimport path not correct,please check.';
                }
                return true;
            }
        },
        backupPath: {
            type: "string",
            description: 'Please enter the backup path',
            'default': mocoolka_function_1.constant('f:/history/backup/tick'),
            componentType: 'string',
            validate(value) {
                if (!mocoolka_tools_io_1.default.directoryExist(path.resolve(value))) {
                    return 'backup path not correct,please check.';
                }
                return true;
            }
        }
    }
};
exports.default = AppOption;
//# sourceMappingURL=config.js.map