var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const mocoolka_tools_io_1 = require("mocoolka-tools-io");
const mocoolka_type_1 = require("mocoolka-type");
const mocoolka_function_1 = require("mocoolka-function");
const mocoolka_cli_1 = require("mocoolka-cli");
const { logo } = mocoolka_cli_1.propComponents;
const { display } = mocoolka_cli_1.components;
const { editRecord } = mocoolka_cli_1.schemaComponent;
const mocoolka_algebraic_1 = require("mocoolka-algebraic");
const config_1 = require("./config");
const { Prism, Lens } = mocoolka_algebraic_1.monocle;
const { fromPredicate } = mocoolka_algebraic_1.option;
/**
 * format number
 * @param n
 */
const formatNumber = (n) => n.toFixed(2).replace(/(\d)(?=(\d{3})+\.)/g, '$1,');
/**
 * Csv to mongoDB App
 */
class App {
    /**
     * create a instance
     * @param _isWatch
     */
    constructor() {
        this.validImport = (thisArg, flags) => {
            mocoolka_algebraic_1.either.fromOption(null)(mocoolka_algebraic_1.option.fromNullable(flags.import)).fold(() => {
                thisArg.start(true);
            }, () => { thisArg.start(false); });
        };
        this.validReset = (thisArg, flags) => {
            mocoolka_algebraic_1.either.fromOption(flags)(mocoolka_algebraic_1.option.fromNullable(flags.reset)).fold(() => thisArg.validImport(thisArg, flags), () => init(thisArg.options));
        };
        this.argv = (flags) => {
            this.validReset(this, flags);
        };
        /**
         * start import
         */
        this.start = (isWatch) => __awaiter(this, void 0, void 0, function* () {
            this.options.isWatch = isWatch;
            this.isWatch = isWatch;
            process.stdout.write(logo('logo')(this.options));
            console.log(`\n`);
            console.log(display `{green.bold  ${this.options.title}}`);
            if (this.options.isWatch)
                watchText();
            this.run();
        });
        this.calRate = (rate) => {
            this.totalTime += rate.total;
            this.size += rate.size;
            this.speedRate = this.size / this.totalTime;
        };
        this.startImport = () => __awaiter(this, void 0, void 0, function* () {
            this.appConfig = mocoolka_tools_io_1.default.openFile(this.options.file);
            const result = yield execute(this.options, this.appConfig, this.speedRate);
            this.calRate(result);
        });
        /**
         * running import program
         */
        this.run = () => __awaiter(this, void 0, void 0, function* () {
            if (this.isWatch)
                this.stopWatch();
            const prism = Prism.fromPredicate(mocoolka_tools_io_1.default.fileExist);
            yield prism.getOption(this.options.file).fold(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield init(this.options);
                yield result.map(this.startImport);
            }), () => __awaiter(this, void 0, void 0, function* () {
                yield this.startImport();
            }));
            if (this.isWatch) {
                // console.log(display`{yellow.bold watching start.Press Ctrl+C quit. }`)
                this.startWatch();
            }
        });
        /**
         * stop watch
         */
        this.stopWatch = () => {
            if (this.timer)
                clearInterval(this.timer);
        };
        /**
         * start watch
         */
        this.startWatch = () => {
            this.timer = setInterval(function (thisArg) {
                thisArg.run();
            }, 1000, this);
        };
        this.options = config_1.default;
        this.options.isWatch = this.options.isWatch || false;
        this.isWatch = this.options.isWatch;
        this.totalTime = 0;
        this.size = 0;
    }
    /**
     * instead of new App
     * @return {App}
     */
    static of() {
        return new App();
    }
}
/**
 * execute when empty directory
 */
const emptyFiles = (options) => __awaiter(this, void 0, void 0, function* () {
    if (!options.isWatch)
        console.log(display ` {yellow.bold No any matched file be found}`);
    return { size: 0, total: 0 };
});
/**
 * build app.json
 */
const init = (options) => __awaiter(this, void 0, void 0, function* () {
    const result = yield editRecord.of(options.params).render();
    return fromPredicate(mocoolka_type_1.isPlainObject)(result).map(function (result) {
        mocoolka_tools_io_1.default.saveFile(options.file, result);
        return result;
    });
});
/**
 * import every file in directory
 */
const execute = (options, appConfig, speedRate) => __awaiter(this, void 0, void 0, function* () {
    let files = mocoolka_tools_io_1.default.findFiles(appConfig.pattern);
    const isEmptyPrism = Prism.fromPredicate(mocoolka_function_1.notPredicate(mocoolka_type_1.isEmpty)).getOption(files);
    return yield isEmptyPrism.fold(() => emptyFiles(options), (files) => __awaiter(this, void 0, void 0, function* () {
        const fileObjects = files.filter(mocoolka_function_1.notPredicate(mocoolka_tools_io_1.default.isLocked)).map(file => ({
            size: fs.statSync(file).size,
            name: path.basename(file),
            path: file
        }));
        const totalSize = fileObjects.reduce((prev, current) => {
            return current.size + prev;
        }, 0);
        const beginTime = Date.now();
        for (let file of fileObjects) {
            yield csvToFile(file, appConfig, options, speedRate);
        }
        const endTime = Date.now();
        console.log(display ` Pattern:  -  {yellow.bold ${fileObjects.length}} matches\n Total size:   {yellow.bold ${formatNumber(totalSize / 1024 / 1024)}MB}\n Done in {yellow.bold ${formatNumber((endTime - beginTime) / 1000 / 60)}minute}`);
        if (options.isWatch)
            watchText();
        return {
            total: (endTime - beginTime),
            size: totalSize
        };
    }));
});
/**
 * import a file to mongoDB
 * @param options
 * @param file
 * @param speedRate
 * @param appConfig
 */
const csvToFile = (file, appConfig, options, speedRate) => __awaiter(this, void 0, void 0, function* () {
    const mongoimport = path.resolve(appConfig.mongoimportPath, 'mongoimport');
    let commandOptions = ['--host', appConfig.hostname, '--port', appConfig.port, '--db',
        appConfig.database, '--collection', appConfig.collection, '--type', 'csv', '--ignoreBlanks', '--file', `${file.path}`];
    const isHeader = Prism.fromPredicate(Lens.fromProp('isHeader').get).getOption(options);
    isHeader.fold(() => commandOptions.push('--fields', options.fields), () => commandOptions.push(`--headerline`));
    const time = (speedRate > 0) ? formatNumber(file.size / speedRate / 1000 / 60) : '?';
    console.log(display `\n start import file {green.bold ${file.name}}({green.bold ${formatNumber(file.size / 1024 / 1024)}MB}) estimate {green.bold ${time}minute}.\n`);
    yield mocoolka_cli_1.execCommandFile(mongoimport, commandOptions, process.stdout, '      ');
    try {
        fs.renameSync(file.path, path.resolve(appConfig.backupPath, 'B' + Date.now() + '_' + file.name));
        console.log(display `\n The file be moved to {yellow.bold ${appConfig.backupPath}}\n`);
    }
    catch (error) {
        if (error && error.code === 'EBUSY') {
            console.log(display `\n {red.bold The file be locked and can't move} \n`);
        }
        else {
            console.error(error);
        }
    }
});
const watchText = () => console.log(display ` Watching start.Press  {yellow.bold Ctrl+C} quit.`);
exports.default = App;
//# sourceMappingURL=index.js.map